package com.yxproject.start.mapper;

import com.yxproject.start.entity.*;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

/**
 * Created by zhangyusheng on 2018/10/13 9:36
 */
@Mapper
public interface TaskMapper {
    public List<TaskEntity> findTaskEntity(long id);
    public int insertTaskEntity(TaskEntity taskEntity);
    public  int updateTaskEntity(TaskEntity taskEntity);
    public  List<TaskEntity> findTaskEntityByAcceptNo(String acceptNo);
    public List<TaskEntity> findTaskEntityByState(int state);
    public int replaceExceptionInformation(TaskEntity taskEntity);
    public int totalNum (int i);
    public List<TypeListEntity> findTypeListByTaskStateId(String taskStateId);
    public List <CityListEntity> findCityList(@Param("taskStateId") String taskStateId,@Param("cardType") String cardType);
    public int updateOutStorageDate( @Param("taskListId")String taskId, @Param("inStorageCount")String inStorageCount, @Param("outStorageCount")String outStorageCount);
    public int updateInStorageDate(String taskListId,String inStorageCount,String outStorageCount);
}
